/*
*
*    RaphaëlSM : Raphaël extension for civil engineering mechanics drawings 
*                                                                             
*                            version 0.8 (2014-11-04)
*  
*                    Copyright (C)  2011 - 2014  Jan Stransky                 
*  
*           Czech Technical University, Faculty of Civil Engineering,         
*       Department of Structural Mechanics, 166 29 Prague, Czech Republic     
*  
*  RaphaëlSM is free software: you can redistribute it and/or modify it under
*  the terms of the GNU Lesser General Public License as published by the Free
*  Software Foundation, either version 3 of the License, or (at your option)
*  any later version.
*  
*  RaphaëlSM is distributed in the hope that it will be useful, but WITHOUT
*  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
*  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
*  for more details.
*  
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/


/**
* @fileOverview <a href="http://raphaeljs.com/">Raphaël</a> (svg javascript library) extension by functions and "classes" for structural mechanics drawings (supports, deflections etc.). Requires <a href="http://raphaeljs.com/">raphael.js</a> and <a href="http://mech.fsv.cvut.cz/~stransky/software/raphaeltools/">raphaeltools.js</a> loaded before loading this file. For more information see <a href="http://mech.fsv.cvut.cz/~stransky/software/raphaelsm/">project homepage</a>.
<br /><br/>RaphaëlSM is a free software distributed under <a href='http://www.gnu.org/licenses/gpl.html'>GNU GPL license</a>.
* @author <a href="http://mech.fsv.cvut.cz/~stransky">Jan Stránský</a>
* @version Version 0.8 (2014-11-04)
*/




/** Global namespace object
* @namespace
*/
var RaphaelSM = {};





/** Abstract class for simple geometrical objects [Raphael.path]
* @class Abstract class for geometrical object
* @extends Raphael.fn
* @property {number} x x coordinate of the object
* @property {number} y y coordinate of the object
* @property {number} size size of the object
* @property {number} angle angle of the object
* @property {Array} pathList svg array (list) of the object
* @param {Object=} params, see below
* @param {number=} params.x x coordinate [pixel]
* @param {number=} params.y y coordinate [pixel]
* @param {number=} params.size size [pixel]
* @param {number=} params.angle rotation angle from initial position
* @this Raphael.fn.GeomObj
* @constructor
*/
Raphael.fn.GeomObj = function(params) {
	params = params==undefined? {} : params;
	/** @alias Raphael.fn.GeomObj.prototype */
	var retGeomObj = this.path();
	retGeomObj.x = params.x==undefined? 0. : params.x;
	retGeomObj.y = params.y==undefined? 0. : params.y;
	retGeomObj.size = params.size==undefined? 1. : params.size;
	retGeomObj.angle = params.angle==undefined? 0. : params.angle;
	retGeomObj.pathList = null;

	retGeomObj.copies = [];
	retGeomObj.copiesShifts = [];

	/** Compute sin from stored (anti-clockwise) angle
	*/
	retGeomObj.sin = function() {
		return Math.sin(-Raphael.rad(this.angle));
	}

	/** Compute cos from stored angle
	*/
	retGeomObj.cos = function() {
		return Math.cos(-Raphael.rad(this.angle));
	}

	/** Set new this.pathList of receiver (each derived class must define its own setPathList method)
	*/
	retGeomObj.setPathList = function() {}

	/** Sets new x and y position of receiver (this.x = x; this.y = y)
	* @param {number} x new x coordinate [pixel]
	* @param {number} y new y coordinate [pixel]
	*/
	retGeomObj.setXY = function(x,y) {
		this.x = x;
		this.y = y;
		this.update();
	}

	/** Sets new x position of receiver (this.x = x)
	* @param {number} x new x coordinate [pixel]
	*/
	retGeomObj.setX = function(x) {
		this.x = x;
		this.update();
	}

	/** Sets new y position of receiver (this.y = y)
	* @param {number} y new y coordinate [pixel]
	*/
	retGeomObj.setY = function(y) {
		this.y = y;
		this.update();
	}

	/** Translate receiver in x and y direction (this.x += dx; this.y += dy)
	* @param {number} dx distance to be translated in x direction [pixel]
	* @param {number} dy distance to be translated in y direction [pixel]
	*/
	retGeomObj.translateXY = function(dx,dy) {
		this.x += dx;
		this.y += dy;
		this.update();
	}

	/** Translate receiver in x direction (this.x += x)
	* @param {number} dx distance to be translated [pixel]
	*/
	retGeomObj.translateX = function(dx) {
		this.translateXY(dx,0.);
	}

	/** Translate receiver in x direction (this.y += dy)
	* @param {number} dy distance to be translated [pixel]
	*/
	retGeomObj.translateY = function(dy) {
		this.translateXY(0.,dy);
	}

	/** Rotate receiver by given angle (this.angle += phi)
	* @param {number} phi angle of rotation
	*/
	retGeomObj.rotate = function(phi) {
		this.angle += phi;
		this.update();
	}

	/** Set angle of receiver (this.angle = phi)
	* @param {number} phi angle of rotation
	*/
	retGeomObj.setAngle = function(phi) {
		this.angle = phi;
		this.update();
	}

	/** Copy receiver
	* @param {Array.<Array.<number>>} shifts array of [x,y] couple of copies shifts
	*/
	retGeomObj.copy = function(shifts) {
		for (var i=0; i<shifts.length; i++) {
			this.copiesShifts.push(shifts[i]);
			this.copies.push(this.makeCopy());
			this.update();
		}
	}

	/** Make a copy of itself (each derived class to be copied have to define its own makeCopy method)
	*/
	retGeomObj.makeCopy = function() {}

	/** Update receiver (set this.pathList for given internal variables - x,y,angle... - as well as its copies)
	*/
	retGeomObj.update = function() {
		this.setPathList();
		this.attr({"path":this.pathList});
		for (var c=0; c<this.copies.length; c++) {
			this.copies[c].setXY(this.x+this.copiesShifts[c][0],this.y+this.copiesShifts[c][1]);
		}
	}

	return retGeomObj;
}





/** Arrow [GeomObj]
* @class represents arrow
* @extends Raphael.fn.GeomObj
* @param {Object=} params parameters, see below
* @param {number=} params.x =0.] x coordinate of arrow tip [pixel]
* @param {number=} params.y =0.] y coordinate of arrow tip [pixel]
* @param {number=} params.size =100.] length of the whole arrow (its size) [pixel]
* @param {number=} params.angle =0.] the arrow will be rotated by this angle [deg]
* @param {number=} params.style =0] style of the arrow head: 0 (default) = similar to svg arrow, 1 = simple
* @param {number=} params.headLen =size/4.] length of the arrow head [pixel]
* @param {number=} params.headWidth =size/16.] width of the arrow head [pixel]
* @property {number} headLen length of arrow head [pixel]
* @property {number} headWidth width of arrow head [pixel]
* @property {number} style style of arrow head
* @borrows Raphael.fn.GeomObj.copy as this.copy
* @borrows Raphael.fn.GeomObj.cos as this.cos
* @borrows Raphael.fn.GeomObj.sin as this.sin
* @borrows Raphael.fn.GeomObj.setX as this.setX
* @borrows Raphael.fn.GeomObj.setY as this.setY
* @borrows Raphael.fn.GeomObj.setXY as this.setXY
* @borrows Raphael.fn.GeomObj.translateX as this.translateX
* @borrows Raphael.fn.GeomObj.translateY as this.translateY
* @borrows Raphael.fn.GeomObj.translateXY as this.translateXY
* @borrows Raphael.fn.GeomObj.rotate as this.rotate
* @borrows Raphael.fn.GeomObj.setAngle this.setAngle
* @borrows Raphael.fn.GeomObj.update as this.update
* @borrows Raphael.fn.GeomObj#x as this.x
* @borrows Raphael.fn.GeomObj#y as this.y
* @borrows Raphael.fn.GeomObj#size as this.size
* @borrows Raphael.fn.GeomObj#angle as this.angle
* @borrows Raphael.fn.GeomObj#pathList as this.pathList
* @this Raphael.fn.Arrow
* @constructor
*/
Raphael.fn.Arrow = function(params) {
	params = params==undefined? {} : params;
	/** @alias Raphael.fn.Arrow.prototype */
	var retArrow = this.GeomObj(params);//.attr({"stroke-linejoin":"miter","stroke-miterlimit":1000});

	if (params.size==undefined) { retArrow.size = 50.; }
	retArrow.headLen = params.headLen==undefined? retArrow.size/4. : params.headLen;
	retArrow.headWidth = params.headWidth==undefined? retArrow.size/16. : params.headWidth;
	retArrow.style = params.style==undefined? 0 : params.style;

	/** Sets svg path of receiver
	*/
	retArrow.setPathList = function() {
		var sin = this.sin();
		var cos = this.cos();
		var t = 0.8;
		switch (this.style) {
			case 1:
				this.pathList = [
					["M",this.x,this.y],
					["L",this.x+cos*this.size,this.y+sin*this.size],
					["M",this.x+cos*this.headLen-sin*this.headWidth,this.y+sin*this.headLen+cos*this.headWidth],
					["L",this.x,this.y],
					["L",this.x+cos*this.headLen+sin*this.headWidth,this.y+sin*this.headLen-cos*this.headWidth]
				];
				break
			default:
				this.pathList = [
					["M",this.x+cos*this.headLen*t,this.y+sin*this.headLen*t],
					["L",this.x+cos*this.size,this.y+sin*this.size],
					["M",this.x+cos*this.headLen*t,this.y+sin*this.headLen*t],
					["L",this.x+cos*this.headLen-sin*this.headWidth,this.y+sin*this.headLen+cos*this.headWidth],
					["L",this.x,this.y],
					["L",this.x+cos*this.headLen+sin*this.headWidth,this.y+sin*this.headLen-cos*this.headWidth],
					["Z"]
				];
		}
	}

	/** Resize receiver in both directions
	* @param {number} newSize new size of receiver
	*/
	retArrow.resize = function(newSize) {
		this.show();
		if (Math.abs(newSize)<1) {
			this.hide();
			return;
		}
		this.headLen *= newSize/this.size;
		this.headWidth *= newSize/this.size;
		this.size = newSize;
		this.update();
	}

	/** Returns copy of receiver
	* @returns {Raphael.fn.Arrow} copy of receiver
	*/
	retArrow.makeCopy = function() {
		var ret2 = this.paper.Arrow({x:this.x,y:this.y,size:this.size,angle:this.angle,headLen:this.headLen,headWidth:this.headWidth,style:this.style});
		ret2.attr(this.attr());
		ret2.update()
		return ret2;
	}

	retArrow.update();
	return retArrow;
}




/** Pin support [GeomObj]
* @class represents pin support
* @extends Raphael.fn.GeomObj
* @borrows Raphael.fn.GeomObj.copy as this.copy
* @borrows Raphael.fn.GeomObj.cos as this.cos
* @borrows Raphael.fn.GeomObj.sin as this.sin
* @borrows Raphael.fn.GeomObj.setX as this.setX
* @borrows Raphael.fn.GeomObj.setY as this.setY
* @borrows Raphael.fn.GeomObj.setXY as this.setXY
* @borrows Raphael.fn.GeomObj.translateX as this.translateX
* @borrows Raphael.fn.GeomObj.translateY as this.translateY
* @borrows Raphael.fn.GeomObj.translateXY as this.translateXY
* @borrows Raphael.fn.GeomObj.rotate as this.rotate
* @borrows Raphael.fn.GeomObj.setAngle this.setAngle
* @borrows Raphael.fn.GeomObj.update as this.update
* @borrows Raphael.fn.GeomObj#x as this.x
* @borrows Raphael.fn.GeomObj#y as this.y
* @borrows Raphael.fn.GeomObj#size as this.size
* @borrows Raphael.fn.GeomObj#angle as this.angle
* @borrows Raphael.fn.GeomObj#pathList as this.pathList
* @param {Object=} params parameters, see below
* @param {number=} params.x =0.] x coordinate of supprt tip [pixel]
* @param {number=} params.y =0.] y coordinate of supprt tip [pixel]
* @param {number=} params.size =20.] height of the support [pixel]
* @param {number=} params.angle =0.] the support will be rotated by angle [deg]
* @param {boolean=} params.sliding =true] if the support is sliding or not
* @param {number=} params.gap =5.] gap between triangle and line (sliding symbol) [pixel]
* @param {number=} params.ratio =0.7] width/height of the triangle [-]
* @property {boolean} sliding true for sliding pins and false for fixed pins
* @property {number} ratio width/height of the pin
* @property {boolean} sliding true for sliding pins and false for fixed pins
* @this Raphael.fn.PinSupport
* @constructor
*/
Raphael.fn.PinSupport = function(params) {
	params = params==undefined? {} : params;
	/** @alias Raphael.fn.PinSupport.prototype */
	var retPinSupport = this.GeomObj(params);

	if (params.size==undefined) { retPinSupport.size = 20.; }
	retPinSupport.sliding = params.sliding==undefined? false : params.sliding
	retPinSupport.ratio = params.ratio==undefined? .7 : params.ratio;
	retPinSupport.gap = params.gap==undefined? 5. : params.gap;

	/** Sets svg path of receiver
	*/
	retPinSupport.setPathList = function() {
		var sin = this.sin();
		var cos = this.cos();
		if (this.sliding) {
			this.pathList = [
				["M",
					this.x,
					this.y],
				["L",
					this.x - cos*this.ratio*this.size - sin*this.size,
					this.y - sin*this.ratio*this.size + cos*this.size],
				["L",
					this.x + cos*this.ratio*this.size - sin*this.size,
					this.y + sin*this.ratio*this.size + cos*this.size],
				["Z"],
				["M",
					this.x - cos*this.ratio*this.size - sin*(this.size+this.gap),
					this.y - sin*this.ratio*this.size + cos*(this.size+this.gap)],
				["L",
					this.x + cos*this.ratio*this.size - sin*(this.size+this.gap),
					this.y + sin*this.ratio*this.size + cos*(this.size+this.gap)]];
		}
		else {
			this.pathList = [
				["M",
					this.x,
					this.y],
				["L",
					this.x - cos*this.ratio*this.size - sin*this.size,
					this.y - sin*this.ratio*this.size + cos*this.size],
				["L",
					this.x + cos*this.ratio*this.size - sin*this.size,
					this.y + sin*this.ratio*this.size + cos*this.size],
				["Z"]];
		}
	}

	/** Returns copy of receiver
	* @returns {Raphael.fn.PinSupport} copy of receiver
	*/
	retPinSupport.makeCopy = function() {
		var ret2 = this.paper.PinSupport({x:this.x,y:this.y,size:this.size,angle:this.angle,sliding:this.sliding,gap:this.gap,ratio:this.ratio});
		ret2.attr(this.attr());
		return ret2;
	}

	retPinSupport.update();
	return retPinSupport;
}



/** Fixed support 
* @class represents fixed support
* @extends Raphael.fn.GeomObj
* @borrows Raphael.fn.GeomObj.copy as this.copy
* @borrows Raphael.fn.GeomObj.cos as this.cos
* @borrows Raphael.fn.GeomObj.sin as this.sin
* @borrows Raphael.fn.GeomObj.setX as this.setX
* @borrows Raphael.fn.GeomObj.setY as this.setY
* @borrows Raphael.fn.GeomObj.setXY as this.setXY
* @borrows Raphael.fn.GeomObj.translateX as this.translateX
* @borrows Raphael.fn.GeomObj.translateY as this.translateY
* @borrows Raphael.fn.GeomObj.translateXY as this.translateXY
* @borrows Raphael.fn.GeomObj.rotate as this.rotate
* @borrows Raphael.fn.GeomObj.setAngle this.setAngle
* @borrows Raphael.fn.GeomObj.update as this.update
* @borrows Raphael.fn.GeomObj#x as this.x
* @borrows Raphael.fn.GeomObj#y as this.y
* @borrows Raphael.fn.GeomObj#size as this.size
* @borrows Raphael.fn.GeomObj#angle as this.angle
* @borrows Raphael.fn.GeomObj#pathList as this.pathList
* @param {Object=} params parameters, see below
* @param {number=} params.x =0.] x coordinate of the support center [pixel]
* @param {number=} params.y =0.] y coordinate of the support center [pixel]
* @param {number=} params.size =30.] size (height) of the support [pixel]
* @param {number=} params.angle =0.] the support will be rotated by angle [deg]
* @param {number=} params.ratio =2.] height/width (width of the 'flags')
* @param {number=} params.nFlags =5] number of 'flags'
* @param {boolean=} params.reversed =false] reversed direction of 'flags'
* @property {number} ratio ratio between length of support and 'flag'
* @property {number} nFlags number of flags
* @property {boolean} reversed if flags are reversed or not
* @this Raphael.fn.FixedSupport
* @constructor
*/
Raphael.fn.FixedSupport = function(params) {
	params = params==undefined? {} : params;
	/** @alias Raphael.fn.FixedSupport.prototype */
	var retFixedSupport = this.GeomObj(params);

	if (params.size==undefined) { retFixedSupport.size = 30.; }
	retFixedSupport.ratio = params.ratio==undefined? .2 : params.ratio;
	retFixedSupport.nFlags = params.nFlags==undefined? 5 : params.nFlags;
	retFixedSupport.reversed = params.reversed==undefined? false : params.reversed;

	/** Sets svg path of receiver
	*/
	retFixedSupport.setPathList = function() {
		var sin = this.sin();
		var cos = this.cos();
		this.pathList = [
			["M",
				this.x - sin*this.size*.5,
				this.y + cos*this.size*.5],
			["L",
				this.x + sin*this.size*.5,
				this.y - cos*this.size*.5]];
		if (this.reversed) {
			for (var i=0; i<this.nFlags; i++) {
				var pt0 = RaphaelTools.vecTrans(0,this.size*(-.5+(i+1)/this.nFlags),sin,cos);
				var pt1 = RaphaelTools.vecTrans(-this.size*this.ratio,this.size*(-.5+i/this.nFlags),sin,cos);
				this.pathList.push(["M",this.x+pt0[0],this.y+pt0[1]]);
				this.pathList.push(["L",this.x+pt1[0],this.y+pt1[1]]);
			}
		}
		else {
			for (var i=0; i<this.nFlags; i++) {
				var pt0 = RaphaelTools.vecTrans(0,this.size*(-.5+i/this.nFlags),sin,cos);
				var pt1 = RaphaelTools.vecTrans(-this.size*this.ratio,this.size*(-.5+(i+1)/this.nFlags),sin,cos);
				this.pathList.push(["M",this.x+pt0[0],this.y+pt0[1]]);
				this.pathList.push(["L",this.x+pt1[0],this.y+pt1[1]]);
			}
		}
	}

	/** Returns copy of receiver
	* @returns {Raphael.fn.FixedSupport} copy of receiver
	*/
	retFixedSupport.makeCopy = function() {
		var ret2 = this.paper.FixedSupport({x:this.x,y:this.y,size:this.size,angle:this.angle,ratio:this.ratio,nFlags:this.nFlags,reversed:this.reversed});
		ret2.attr(this.attr())
		ret2.update();
		return ret2;
	}

	retFixedSupport.update();
	return retFixedSupport;
}



/** Distributed load [Raphael.path]
* @class represents distributed load
* @extends Raphael.fn
* @param {RaphaelTools.Point2d} point1 1st point
* @param {RaphaelTools.Point2d} point2 2nd point
* @param {Object=} params parameters, see below
* @param {number=} params.valx =0.] value of load in x direction (global)
* @param {number=} params.valy =0.] value of load in y direction (global)
* @param {number=} params.nArrows =4] number of arrows
* @param {number=} params.style =0] style of the arrow head: 0 (default) = similar to svg arrow, 1 = simple
* @param {boolean=} params.cloased =false] if the beam line is present or not
* @property {Point2d} point1 1st point
* @property {Point2d} point2 2nd point
* @property {number} valx value in global x direction
* @property {number} valy value in global y direction
* @property {number} nArrows number of displayed arrows
* @property {number} style
* @this Raphael.fn.DistributedLoad
* @constructor
*/
Raphael.fn.DistributedLoad = function(point1,point2,params) {
	/** @alias Raphael.fn.DistributedLoad.prototype */
	var retDistributedLoad = this.path();
	retDistributedLoad.point1 = point1;
	retDistributedLoad.point2 = point2;
	params = params==undefined? {} : params;
	retDistributedLoad.valx = params.valx==undefined? 0. : params.valx;
	retDistributedLoad.valy = params.valy==undefined? 0. : params.valy;
	retDistributedLoad.nArrows = params.nArrows==undefined? 4 : params.nArrows;
	retDistributedLoad.style = params.style==undefined? 0 : params.style;
	retDistributedLoad.closed = params.closed==undefined? false : params.closed;

	/** Sets new values to receiver
	* @param {number=} valx =0.] value in global x direction
	* @param {number=} valy =0.] value in global y direction
	*/
	retDistributedLoad.setVals = function(valx,valy) {
		this.valx = valx==undefined? 0. : valx;
		this.valy = valy==undefined? 0. : valy;
		this.update();
	}

	/** Sets svg path of receiver
	*/
	retDistributedLoad.setPathList = function() {
		// set geom
		var x1 = this.point1.x;
		var y1 = this.point1.y;
		var x2 = this.point2.x;
		var y2 = this.point2.y;
		var dx = x2 - x1;
		var dy = y2 - y1;
		var len = Math.sqrt(dx*dx+dy*dy);
		var cos = dx/len;
		var sin = dy/len;
		var lenVals = Math.sqrt(this.valx*this.valx+this.valy*this.valy);
		if (Math.abs(lenVals) < 1) {
			this.hide();
			return;
		} else {
			this.show();
		}
		var cosVal = this.valx/lenVals;
		var sinVal = this.valy/lenVals;
		//
		this.pathList = [
			["M",x1,y1],
			["L",x1-this.valx,y1-this.valy],
			["M",x1-this.valx,y1-this.valy],
			["L",x2-this.valx,y2-this.valy],
			["M",x2-this.valx,y2-this.valy],
			["L",x2,y2]
		];
		if (this.closed) {
			this.pathList.push(["M",x2,y2]);
			this.pathList.push(["L",x1,y1]);
		}
		var nArrows = Math.round(this.nArrows)
		var segLen = len/(nArrows+1);
		var t = .8;
		for (var i=1; i<=nArrows; i++) {
			switch (this.style) {
				case 1:
					this.pathList.push(["M",
							x1 - this.valx + i*segLen*cos,
							y1 - this.valy + i*segLen*sin],
						["L",x1 + i*segLen*cos, y1 + i*segLen*sin],
						["M",
							x1 + i*segLen*cos - cosVal*lenVals/4. + sinVal*lenVals/16.,
							y1 + i*segLen*sin - sinVal*lenVals/4. - cosVal*lenVals/16.],
						["L",x1 + i*segLen*cos, y1 + i*segLen*sin],
						["L",
							x1 + i*segLen*cos - cosVal*lenVals/4. - sinVal*lenVals/16.,
							y1 + i*segLen*sin - sinVal*lenVals/4. + cosVal*lenVals/16.]
					);
					break;
				default:
					this.pathList.push(["M",
							x1 - this.valx + i*segLen*cos,
							y1 - this.valy + i*segLen*sin],
						["L",
							x1 + i*segLen*cos - t*cosVal*lenVals/4.,
							y1 + i*segLen*sin - t*sinVal*lenVals/4.],
						["M",
							x1 + i*segLen*cos - t*cosVal*lenVals/4.,
							y1 + i*segLen*sin - t*sinVal*lenVals/4.],
						["L",
							x1 + i*segLen*cos - cosVal*lenVals/4. + sinVal*lenVals/16.,
							y1 + i*segLen*sin - sinVal*lenVals/4. - cosVal*lenVals/16.],
						["L",x1 + i*segLen*cos, y1 + i*segLen*sin],
						["L",
							x1 + i*segLen*cos - cosVal*lenVals/4. - sinVal*lenVals/16.,
							y1 + i*segLen*sin - sinVal*lenVals/4. + cosVal*lenVals/16.],
						["Z"]
					);
			}
		}
	}

	/** Update receiver (set this.pathList for given internal variables)
	*/
	retDistributedLoad.update = function() {
		this.setPathList();
		this.attr({"path":this.pathList});
	}

	retDistributedLoad.update();
	return retDistributedLoad;
}





/** Abstract class for curves (applied as internal forces, deflection etc.)
* @class represents generic curve object drawn over a line (e.g. beam)
* @extends Raphael.fn
* @param {RaphaelTools.Point2d} point1 first point of curve
* @param {RaphaelTools.Point2d} point2 second point of curve
* @param {Object=} params parameters, see below
* @param {number=} params.scale =1.] scale scale between passed value and drawn value in pixels
* @param {boolean=} params.closedBegin =true] if point1 is connected with appropriate value of curve
* @param {boolean=} params.closedEnd =true] if point2 is connected with appropriate value of curve
* @param {boolean=} params.closedBase =true] if point1 and point2 are connected within the curve definition
* @property {Point2d} point1 1st point
* @property {Point2d} point2 2nd point
* @property {number} scale scale scale between passed value and drawn value in pixels
* @property {boolean} closedBegin if point1 is connected with appropriate value of curve
* @property {boolean} closedEnd if point2 is connected with appropriate value of curve
* @property {boolean} closedBase if point1 and point2 are connected within the curve definition
* @this Raphael.fn.CurveObj
* @constructor
*/
Raphael.fn.CurveObj = function(point1,point2,params) {
	params = params==undefined? {} : params;
	/** @alias Raphael.fn.CurveObj.prototype */
	var retCurveObj = this.path();
	retCurveObj.point1 = point1;
	retCurveObj.point2 = point2;
	retCurveObj.scale = params.scale==undefined? 1. : params.scale;
	retCurveObj.closedBegin = params.closedBegin==undefined? true : params.closedBegin;
	retCurveObj.closedEnd = params.closedEnd==undefined? true : params.closedEnd;
	retCurveObj.closedBase = params.closedBase==undefined? true : params.closedBase;
	retCurveObj.curve = this.path().hide();

	/** Sets function constants
	*/
	retCurveObj.consts = function() {}

	/** Sets svg path list
	*/
	retCurveObj.setPathList = function() {}

	/** Sets geometric parameters according to stored attributes
	*/
	retCurveObj.setGeom = function() {
		this.x1 = this.point1.x;
		this.y1 = this.point1.y;
		this.x2 = this.point2.x;
		this.y2 = this.point2.y;
		var dx = this.x2-this.x1;
		var dy = this.y2-this.y1;
		this.len = Math.sqrt(dx*dx+dy*dy);
		this.segLen = this.len/(this.nSeg) || 1.
		if (Math.abs(this.len)<1e-12) {
			this.cos = 1.;
			this.sin = 0.;
		} else {
			this.cos = dx/this.len;
			this.sin = dy/this.len;
		}
	}

	/** Update*/
	retCurveObj.update = function() {
		this.setGeom();
		this.consts();
		this.setPathList();
		this.attr({"path":this.pathList});
		this.curve.attr({"path":this.pathList});
	}

	retCurveObj.closedPartsHandle = function(val1) {
		if (this.closedEnd) { this.pathList.push(["L",this.x2,this.y2]); }
		else { this.pathList.push(["M",this.x2,this.y2]); }
		if (this.closedBase) { this.pathList.push(["L",this.x1,this.y1]); }
		else { this.pathList.push(["M",this.x1,this.y1]); }
		if (this.closedBegin) { this.pathList.push(["L",this.x1-this.scale*this.sin*val1,this.y1+this.scale*this.cos*val1]); }
	}

	retCurveObj.update();
	return retCurveObj;
}




/** Constant curve (e.g. shear force line)
* @class represents constant curve
* @extends Raphael.fn.CurveObj
* @param {RaphaelTools.Point2d} point1 1st point
* @param {RaphaelTools.Point2d} point2 2nd point
* @param {Object=} params parameters, see below
* @param {number=} param.val =0.] value
* @param {number=} param.scale =1.] scale factor
* @param {boolean=} param.closedBegin =true] the line conecting 1st point with 1st nodal value
* @param {boolean=} param.closedEnd =true] the line conecting 2nd point with 2nd nodal value
* @param {boolean=} param.closedBase =true] the line conecting 1st and 2nd point
* @borrows Raphael.fn.CurveObj#point1 as this.point1
* @borrows Raphael.fn.CurveObj#point2 as this.point2
* @borrows Raphael.fn.CurveObj#scale as this.scale
* @borrows Raphael.fn.CurveObj#closedBegin as this.closedBegin
* @borrows Raphael.fn.CurveObj#closedEnd as this.closedEnd
* @borrows Raphael.fn.CurveObj#closedBase as this.closedBase
* @borrows Raphael.fn.CurveObj.setPathList as this.setPathList
* @borrows Raphael.fn.CurveObj.consts as this.consts
* @borrows Raphael.fn.CurveObj.update as this.update
* @property {number} val value in 1st point
* @this Raphael.fn.ConstantCurve
* @constructor
*/
Raphael.fn.ConstantCurve = function(point1,point2,params) {
	params = params==undefined? {} : params;
	/** @alias Raphael.fn.ConstantCurve.prototype */
	var retConstantCurve = this.CurveObj(point1,point2,params);

	retConstantCurve.val = params.val==undefined? 0. : params.val;

	retConstantCurve.setPathList = function() {
		this.pathList =   [["M",
				this.x1 - this.scale*this.sin*this.val,
				this.y1 + this.scale*this.cos*this.val]];
		this.pathList.push(["L",
			this.x2 - this.scale*this.sin*this.val,
			this.y2 + this.scale*this.cos*this.val]);
		this.closedPartsHandle(this.val);
	}

	/** Sets value of receiver
	* @param {number} val value in 1st point
	*/
	retConstantCurve.setVal = function(val) {
		this.val = val;
		this.update();
	}

	retConstantCurve.update();
	return retConstantCurve;
}




/** Linear curve (e.g. shear force line)
* @class represents linear curve
* @extends Raphael.fn.CurveObj
* @param {RaphaelTools.Point2d} point1 1st point
* @param {RaphaelTools.Point2d} point2 2nd point
* @param {Object=} params parameters, see below
* @param {number=} param.val1 =0.] value in 1st point
* @param {number=} param.val2 =0.] value in 2nd point
* @param {number=} param.scale =1.] scale factor
* @param {boolean=} param.closedBegin =true] the line conecting 1st point with 1st nodal value
* @param {boolean=} param.closedEnd =true] the line conecting 2nd point with 2nd nodal value
* @param {boolean=} param.closedBase =true] the line conecting 1st and 2nd point
* @borrows Raphael.fn.CurveObj#point1 as this.point1
* @borrows Raphael.fn.CurveObj#point2 as this.point2
* @borrows Raphael.fn.CurveObj#scale as this.scale
* @borrows Raphael.fn.CurveObj#closedBegin as this.closedBegin
* @borrows Raphael.fn.CurveObj#closedEnd as this.closedEnd
* @borrows Raphael.fn.CurveObj#closedBase as this.closedBase
* @borrows Raphael.fn.CurveObj.setPathList as this.setPathList
* @borrows Raphael.fn.CurveObj.consts as this.consts
* @borrows Raphael.fn.CurveObj.update as this.update
* @property {number} val1 value in 1st point
* @property {number} val2 value in 2nd point
* @this Raphael.fn.LinearCurve
* @constructor
*/
Raphael.fn.LinearCurve = function(point1,point2,params) {
	params = params==undefined? {} : params;
	/** @alias Raphael.fn.LinearCurve.prototype */
	var retLinearCurve = this.CurveObj(point1,point2,params);

	retLinearCurve.val1 = params.val1==undefined? 0. : params.val1;
	retLinearCurve.val2 = params.val2==undefined? 0. : params.val2;

	retLinearCurve.setPathList = function() {
		this.pathList =   [["M",
				this.x1 - this.scale*this.sin*this.val1,
				this.y1 + this.scale*this.cos*this.val1]];
		this.pathList.push(["L",
			this.x2 - this.scale*this.sin*this.val2,
			this.y2 + this.scale*this.cos*this.val2]);
		this.closedPartsHandle(this.val1);
	}

	/** Sets values of receiver
	* @param {number} val1 value in 1st point
	* @param {number} val2 value in 2nd point
	*/
	retLinearCurve.setVals = function(val1,val2) {
		this.val1 = val1;
		this.val2 = val2;
		this.update();
	}

	retLinearCurve.update()
	return retLinearCurve;
}





/** Parabolic curve (e.g. bending moment line)
* @class represents parabolic curve
* @extends Raphael.fn.CurveObj
* @param {RaphaelTools.Point2d} point1 1st point
* @param {RaphaelTools.Point2d} point2 2nd point
* @param {Object=} params parameters, see below
* @param {number=} params.val1 =0.] value in 1st point
* @param {number=} params.valMid =0.] value in the middle of 1st and 2nd point
* @param {number=} params.val2 =0.] value in 2nd point
* @param {number=} params.sclale =1.] scale factor
* @param {boolean=} params.closedBegin =true] the line conecting 1st point with 1st nodal value
* @param {boolean=} params.closedEnd =true] the line conecting 2nd point with 2nd nodal value
* @param {boolean=} params.closedBase =true] the line conecting 1st and 2nd point
* @param {number=} params.nSeg =5] number of interpolating points
* @borrows Raphael.fn.CurveObj#point1 as this.point1
* @borrows Raphael.fn.CurveObj#point2 as this.point2
* @borrows Raphael.fn.CurveObj#scale as this.scale
* @borrows Raphael.fn.CurveObj#closedBegin as this.closedBegin
* @borrows Raphael.fn.CurveObj#closedEnd as this.closedEnd
* @borrows Raphael.fn.CurveObj#closedBase as this.closedBase
* @borrows Raphael.fn.CurveObj.setPathList as this.setPathList
* @borrows Raphael.fn.CurveObj.consts as this.consts
* @borrows Raphael.fn.CurveObj.update as this.update
* @property {number} val1 value in 1st point
* @property {number} val2 value in 2nd point
* @property {number} valMid value in the middle
* @this Raphael.fn.ParabolicCurve
* @constructor
*/
Raphael.fn.ParabolicCurve = function(point1,point2,params) {
	params = params==undefined? {} : params;
	/** @alias Raphael.fn.ParabolicCurve.prototype */
	var retParabolicCurve = this.CurveObj(point1,point2,params);

	retParabolicCurve.val1 = params.val1==undefined? 0. : params.val1;
	retParabolicCurve.valMid = params.valMid==undefined? 0. : params.valMid;
	retParabolicCurve.val2 = params.val2==undefined? 0. : params.val2;
	retParabolicCurve.nSeg = params.nSeg==undefined? 5 : params.nSeg;

	retParabolicCurve.consts = function() {
		this.c0 = this.val1;
		this.c1 = (-3*this.val1 - this.val2 + 4*this.valMid)/this.len;
		this.c2 = (2*this.val1 + 2*this.val2 - 4*this.valMid)/this.len/this.len;
	}

	retParabolicCurve.v = function(s) {
		return this.c0 + this.c1*s + this.c2*s*s;
	}

	retParabolicCurve.dv = function(s) {
		return this.c1 + 2*this.c2*s;
	}

	retParabolicCurve.setPathList = function() {
		this.pathList = [["M",this.x1-this.scale*this.sin*this.v(0),this.y1+this.scale*this.cos*this.v(0)]];
		var t = .3;
		for (var i=0; i<this.nSeg; i++) {
			var v1 = this.v(i*this.segLen);
			var v2 = this.v((i+1)*this.segLen);
			var dv1 = this.dv(i*this.segLen);
			var dv2 = this.dv((i+1)*this.segLen);
			var vec1 = RaphaelTools.vecTrans((i+t)*this.segLen,this.scale*(v1+dv1*t*this.segLen),this.sin,this.cos);
			var vec2 = RaphaelTools.vecTrans((i+1-t)*this.segLen,this.scale*(v2-dv2*t*this.segLen),this.sin,this.cos);
			var vec3 = RaphaelTools.vecTrans((i+1)*this.segLen,this.scale*v2,this.sin,this.cos);
			var ll = ["C",this.x1+vec1[0],this.y1+vec1[1],this.x1+vec2[0],this.y1+vec2[1],this.x1+vec3[0],this.y1+vec3[1]];
			this.pathList.push(ll);
		}
		this.closedPartsHandle(this.v(0));
	}

	/** Sets values of receiver
	* @param {number} val1 value in 1st point
	* @param {number} val2 value in 2nd point
	* @param {number} valMid value in the middle
	*/
	retParabolicCurve.setVals = function(val1,val2,valMid) {
		this.val1 = val1;
		this.val2 = val2;
		this.valMid = valMid;
		this.update();
	}

	retParabolicCurve.update();
	return retParabolicCurve;
}



/** Generic class for 3rd and 4th order curves (used for deflection)
* @class represents generic class for deflection curve
* @extends Raphael.fn.CurveObj
* @param {RaphaelTools.Point2d} point1 1st point
* @param {RaphaelTools.Point2d} point2 2nd point
* @param {Object=} params parameters, see below
* @param {number=} params.u1 =0.] local x deflection in the 1st point
* @param {number=} params.w1 =0.] local y deflection in the 1st point
* @param {number=} params.phi1 =0.] rotation of the 1st point
* @param {number=} params.u2 =0.] local x deflection in the 2nd point
* @param {number=} params.w2 =0.] local y deflection in the 2nd point
* @param {number=} params.phi2 =0.] rotation of the 2nd point
* @param {number=} params.wMid =0.] local y deflection from continuous load in the middle of the beam
* @param {number=} params.realLen =1.] real length (in units of u and w) of beam
* @param {number=} params.scale =1.] scale parameter for the deflection line
* @param {boolean=} params.closedBegin =true] the line conecting 1st point with 1st nodal value
* @param {boolean=} params.closedEnd =true] the line conecting 2nd point with 2nd nodal value
* @param {boolean=} params.closedBase =true] the line conecting 1st and 2nd point
* @param {number=} params.nSeg =5] number of interpolation points
* @borrows Raphael.fn.CurveObj#point1 as this.point1
* @borrows Raphael.fn.CurveObj#point2 as this.point2
* @borrows Raphael.fn.CurveObj#scale as this.scale
* @borrows Raphael.fn.CurveObj#closedBegin as this.closedBegin
* @borrows Raphael.fn.CurveObj#closedEnd as this.closedEnd
* @borrows Raphael.fn.CurveObj#closedBase as this.closedBase
* @borrows Raphael.fn.CurveObj.setPathList as this.setPathList
* @borrows Raphael.fn.CurveObj.consts as this.consts
* @borrows Raphael.fn.CurveObj.update as this.update
* @property {number} u1 local x displacement of 1st point
* @property {number} w1 local y displacement of 1st point
* @property {number} phi1 rotation of 1st point
* @property {number} u2 local x displacement of 2nd point
* @property {number} w2 local y displacement of 2nd point
* @property {number} phi2 rotation of 2nd point
* @property {number} wMid local y deflection from continuous load in the middle of the beam
* @property {number} realLen real length (in units of u and w) of beam
* @property {number} scale parameter for the deflection line
* @property {number} nSeg number of interpolation points
* @this Raphael.fn.DeflCurve
* @constructor
*/
Raphael.fn.DeflCurve = function(point1,point2,params) {
	params = params==undefined? {} : params;
	/** @alias Raphael.fn.DeflCurve.prototype */
	var retDeflCurve = this.CurveObj(point1,point2,params);

	retDeflCurve.closedBegin = params.closedBegin==undefined? false : params.closedBegin;
	retDeflCurve.closedEnd = params.closedEnd==undefined? false : params.closedEnd;
	retDeflCurve.closedBase = params.closedBase==undefined? false : params.closedBase;

	retDeflCurve.u1 = params.u1==undefined? 0. : params.u1;
	retDeflCurve.w1 = params.w1==undefined? 0. : params.w1;
	retDeflCurve.phi1 = params.phi1==undefined? 0. : params.phi1;
	retDeflCurve.u2 = params.u2==undefined? 0. : params.u2;
	retDeflCurve.w2 = params.w2==undefined? 0. : params.w2;
	retDeflCurve.phi2 = params.phi2==undefined? 0. : params.phi2;
	retDeflCurve.wMid = params.wMid==undefined? 0. : params.wMid;
	retDeflCurve.realLen = params.realLen==undefined? 1. : params.realLen;
	retDeflCurve.nSeg = params.nSeg==undefined? 5 : params.nSeg;
	retDeflCurve.scale = params.scale==undefined? 1. : params.scale;
	var dx = retDeflCurve.point1.x - retDeflCurve.point2.x;
	var dy = retDeflCurve.point1.y - retDeflCurve.point2.y;
	var len = Math.sqrt(dx*dx + dy*dy);
	retDeflCurve.lenScale = retDeflCurve.realLen/len;

	retDeflCurve.consts = function() {
		this.c0 = this.w1;
		this.c1 = -this.phi1*this.lenScale;
		this.c2 = (2*this.phi1+this.phi2)*this.lenScale/this.len + 3/this.len/this.len*(this.w2-this.w1) + 16/this.len/this.len*this.wMid;
		this.c3 = 2/this.len/this.len/this.len*(this.w1-this.w2) - 1/this.len/this.len*this.lenScale*(this.phi1+this.phi2) - 32/this.len/this.len/this.len*this.wMid;
		this.c4 = 16/this.len/this.len/this.len/this.len*this.wMid;
	}

	retDeflCurve.w = function(s) { return this.c0 + this.c1*s + this.c2*s*s + this.c3*s*s*s + this.c4*s*s*s*s; }

	retDeflCurve.dw = function(s) { return this.c1 + 2*this.c2*s + 3*this.c3*s*s + 4*this.c4*s*s*s; }

	retDeflCurve.setPathList = function() {
		this.pathList = [["M",this.x1+this.scale*(this.u1*this.cos-this.sin*this.w(0)),this.y1+this.scale*(this.u1*this.sin+this.cos*this.w(0))]];
		var t = .3;
		for (var i=0; i<this.nSeg; i++) {
			var w1 = this.w(i*this.segLen);
			var w2 = this.w((i+1)*this.segLen);
			var dw1 = this.dw(i*this.segLen);
			var dw2 = this.dw((i+1)*this.segLen);
			var vec1 = RaphaelTools.vecTrans(this.scale*this.u1+(i+t)*(this.segLen+this.scale*(this.u2-this.u1)/this.nSeg),this.scale*(w1+dw1*t*this.segLen),this.sin,this.cos);
			var vec2 = RaphaelTools.vecTrans(this.scale*this.u1+(i+1-t)*(this.segLen+this.scale*(this.u2-this.u1)/this.nSeg),this.scale*(w2-dw2*t*this.segLen),this.sin,this.cos);
			var vec3 = RaphaelTools.vecTrans(this.scale*this.u1+(i+1)*(this.segLen+this.scale*(this.u2-this.u1)/this.nSeg),this.scale*w2,this.sin,this.cos);
			var ll = ["C",this.x1+vec1[0],this.y1+vec1[1],this.x1+vec2[0],this.y1+vec2[1],this.x1+vec3[0],this.y1+vec3[1]];
			this.pathList.push(ll);
		}
		this.closedPartsHandle(this.w(0));
	}

	/** Sets new values of all stored displacements and rotations
	* @param {number} u1 local x deflection in the 1st point
	* @param {number} w1 local y deflection in the 1st point
	* @param {number} phi1 rotation of the 1st point
	* @param {number} u2 local x deflection in the 2nd point
	* @param {number} w2 local y deflection in the 2nd point
	* @param {number} phi2 rotation of the 2nd point
	* @param {number} wMid local y deflection from continuous load in the middle of the beam
	*/
	retDeflCurve.setDsplFull = function(u1,w1,phi1,u2,w2,phi2,wMid) {
		this.u1 = u1;
		this.w1 = w1;
		this.phi1 = phi1;
		this.u2 = u2;
		this.w2 = w2;
		this.phi2 = phi2;
		this.wMid = wMid || 0.;
		this.update();
	}

	retDeflCurve.update();
	return retDeflCurve;
}



/* ***********************************************************
*
* specific higher order curves
*
************************************************************/

/** Cubic deflection (all 6 dofs can be set)
* @class represents cubic deflection, all 6 dofs can be set
* @extends Raphael.fn.DeflCurve
* @param {RaphaelTools.Point2d} point1 1st point
* @param {RaphaelTools.Point2d} point2 2nd point
* @param {Object=} params
* @param {number=} params.u1 =0.] local x deflection in the 1st point
* @param {number=} params.w1 =0.] local y deflection in the 1st point
* @param {number=} params.phi1 =0.] rotation of the 1st point
* @param {number=} params.u2 =0.] local x deflection in the 2nd point
* @param {number=} params.w2 =0.] local y deflection in the 2nd point
* @param {number=} params.phi2 =0.] rotation of the 2nd point
* @param {number=} params.realLen =1.] real length (in units of u and w) of beam
* @param {number=} params.scale =1.] scale parameter for the deflection line
* @param {boolean=} params.closedBegin =true] the line conecting 1st point with 1st nodal value
* @param {boolean=} params.closedEnd =true] the line conecting 2nd point with 2nd nodal value
* @param {boolean=} params.closedBase =true] the line conecting 1st and 2nd point
* @param {number=} params.nSeg =5] number of interpolation points
* @borrows Raphael.fn.CurveObj#point1 as this.point1
* @borrows Raphael.fn.CurveObj#point2 as this.point2
* @borrows Raphael.fn.CurveObj#scale as this.scale
* @borrows Raphael.fn.CurveObj#closedBegin as this.closedBegin
* @borrows Raphael.fn.CurveObj#closedEnd as this.closedEnd
* @borrows Raphael.fn.CurveObj#closedBase as this.closedBase
* @borrows Raphael.fn.CurveObj.setPathList as this.setPathList
* @borrows Raphael.fn.CurveObj.consts as this.consts
* @borrows Raphael.fn.CurveObj.update as this.update
* @property {number} u1 local x displacement of 1st point
* @property {number} w1 local y displacement of 1st point
* @property {number} phi1 rotation of 1st point
* @property {number} u2 local x displacement of 2nd point
* @property {number} w2 local y displacement of 2nd point
* @property {number} phi2 rotation of 2nd point
* @property {number} realLen real length (in units of u and w) of beam
* @property {number} nSeg number of interpolation points
* @this Raphael.fn.DeflCubic
* @constructor
*/
Raphael.fn.DeflCubic = function(point1,point2,params) {
	params = params==undefined? {} : params;
	/** @alias Raphael.fn.DeflCubic.prototype */
	var retDeflCubic = this.DeflCurve(point1,point2,params);
	retDeflCubic.wMid = params.wMid==undefined? 0. : params.wMid;

	/** Sets new values of all stored displacements and rotations
	* @param {number} u1 local x deflection in the 1st point
	* @param {number} w1 local y deflection in the 1st point
	* @param {number} phi1 rotation of the 1st point
	* @param {number} u2 local x deflection in the 2nd point
	* @param {number} w2 local y deflection in the 2nd point
	* @param {number} phi2 rotation of the 2nd point
	*/
	retDeflCubic.setDspl = function(u1,w1,phi1,u2,w2,phi2) {
		this.setDsplFull(u1,w1,phi1,u2,w2,phi2,0);
	}

	retDeflCubic.update();
	return retDeflCubic;
}





/** 4th order deflection
* @class represents cubic deflection, all 6 dofs can be set
* @extends Raphael.fn.DeflCurve
* @param {RaphaelTools.Point2d} point1 1st point
* @param {RaphaelTools.Point2d} point2 2nd point
* @param {Object=} params parameters, see below
* @param {number=} params.u1 =0.] local x deflection in the 1st point
* @param {number=} params.w1 =0.] local y deflection in the 1st point
* @param {number=} params.phi1 =0.] rotation of the 1st point
* @param {number=} params.u2 =0.] local x deflection in the 2nd point
* @param {number=} params.w2 =0.] local y deflection in the 2nd point
* @param {number=} params.phi2 =0.] rotation of the 2nd point
* @param {number=} params.wMid =0.] local y deflection from continuous load in the middle of the beam
* @param {number=} params.realLen =1.] real length (in units of u and w) of beam
* @param {number=} params.scale =1.] scale parameter for the deflection line
* @param {boolean=} params.closedBegin =true] the line conecting 1st point with 1st nodal value
* @param {boolean=} params.closedEnd =true] the line conecting 2nd point with 2nd nodal value
* @param {boolean=} params.closedBase =true] the line conecting 1st and 2nd point
* @param {number=} params.nSeg =5] number of interpolation points
* @borrows Raphael.fn.CurveObj#point1 as this.point1
* @borrows Raphael.fn.CurveObj#point2 as this.point2
* @borrows Raphael.fn.CurveObj#scale as this.scale
* @borrows Raphael.fn.CurveObj#closedBegin as this.closedBegin
* @borrows Raphael.fn.CurveObj#closedEnd as this.closedEnd
* @borrows Raphael.fn.CurveObj#closedBase as this.closedBase
* @borrows Raphael.fn.CurveObj.setPathList as this.setPathList
* @borrows Raphael.fn.CurveObj.consts as this.consts
* @borrows Raphael.fn.CurveObj.update as this.update
* @property {number} u1 local x displacement of 1st point
* @property {number} w1 local y displacement of 1st point
* @property {number} phi1 rotation of 1st point
* @property {number} u2 local x displacement of 2nd point
* @property {number} w2 local y displacement of 2nd point
* @property {number} phi2 rotation of 2nd point
* @property {number} realLen real length (in units of u and w) of beam
* @property {number} wMid local y deflection from continuous load in the middle of the beam
* @property {number} nSeg number of interpolation points
* @this Raphael.fn.Defl4thOrder
* @constructor
*/
Raphael.fn.Defl4thOrder = function(point1,point2,params) {
	params = params==undefined? {} : params;
	/** @alias Raphael.fn.Defl4thOrder.prototype */
	var retDefl4thOrder = this.DeflCurve(point1,point2,params);

	/** Sets new values of all stored displacements and rotations + wMid
	* @param {number} u1 local x deflection in the 1st point
	* @param {number} w1 local y deflection in the 1st point
	* @param {number} phi1 rotation of the 1st point
	* @param {number} u2 local x deflection in the 2nd point
	* @param {number} w2 local y deflection in the 2nd point
	* @param {number} phi2 rotation of the 2nd point
	* @param {number} wMid local y deflection from continuous load in the middle of the beam
	*/
	retDefl4thOrder.setDspl = function(u1,w1,phi1,u2,w2,phi2,wMid) {
		this.setDsplFull(u1,w1,phi1,u2,w2,phi2,wMid);
	}

	retDefl4thOrder.update();
	return retDefl4thOrder;
}


/** Cubic deflection (no axial displacement)
* @class represents cubic deflection (no axial displacement)
* @extends Raphael.fn.DeflCurve
* @param {RaphaelTools.Point2d} point1 1st point
* @param {RaphaelTools.Point2d} point2 2nd point
* @param {Object=} params parameters, see below
* @param {number=} params.w1 =0.] local y deflection in the 1st point
* @param {number=} params.phi1 =0.] rotation of the 1st point
* @param {number=} params.w2 =0.] local y deflection in the 2nd point
* @param {number=} params.phi2 =0.] rotation of the 2nd point
* @param {number=} params.realLen =1.] real length (in units of u and w) of beam
* @param {number=} params.scale =1.] scale parameter for the deflection line
* @param {boolean=} params.closedBegin =true] the line conecting 1st point with 1st nodal value
* @param {boolean=} params.closedEnd =true] the line conecting 2nd point with 2nd nodal value
* @param {boolean=} params.closedBase =true] the line conecting 1st and 2nd point
* @param {number=} params.nSeg =5] number of interpolation points
* @borrows Raphael.fn.CurveObj#point1 as this.point1
* @borrows Raphael.fn.CurveObj#point2 as this.point2
* @borrows Raphael.fn.CurveObj#scale as this.scale
* @borrows Raphael.fn.CurveObj#closedBegin as this.closedBegin
* @borrows Raphael.fn.CurveObj#closedEnd as this.closedEnd
* @borrows Raphael.fn.CurveObj#closedBase as this.closedBase
* @borrows Raphael.fn.CurveObj.setPathList as this.setPathList
* @borrows Raphael.fn.CurveObj.consts as this.consts
* @borrows Raphael.fn.CurveObj.update as this.update
* @property {number} w1 local y displacement of 1st point
* @property {number} phi1 rotation of 1st point
* @property {number} w2 local y displacement of 2nd point
* @property {number} phi2 rotation of 2nd point
* @property {number} realLen real length (in units of u and w) of beam
* @property {number} nSeg number of interpolation points
* @this Raphael.fn.DeflCubicNoU
* @constructor
*/
Raphael.fn.DeflCubicNoU = function(point1,point2,params) {
	params = params==undefined? {} : params;
	params.wMid = 0.;
	params.u1 = 0.;
	params.u2 = 0.;
	/** @alias Raphael.fn.DeflCubicNoU.prototype */
	var retDeflCubicNoU = this.DeflCurve(point1,point2,params);

	/** Sets new values of w and phi in both points
	* @param {number} w1 local y deflection in the 1st point
	* @param {number} phi1 rotation of the 1st point
	* @param {number} w2 local y deflection in the 2nd point
	* @param {number} phi2 rotation of the 2nd point
	*/
	retDeflCubicNoU.setDspl = function(w1,phi1,w2,phi2) {
		this.setDsplFull(0,w1,phi1,0,w2,phi2,0);
	}

	retDeflCubicNoU.update();
	return retDeflCubicNoU;
}




/** 4th order deflection (no axial displacement)
* @class represents 4th order deflection (no axial displacement)
* @extends Raphael.fn.DeflCurve
* @param {RaphaelTools.Point2d} point1 1st point
* @param {RaphaelTools.Point2d} point2 2nd point
* @param {Object=} params parameters, see below
* @param {number=} params.w1 =0.] local y deflection in the 1st point
* @param {number=} params.phi1 =0.] rotation of the 1st point
* @param {number=} params.w2 =0.] local y deflection in the 2nd point
* @param {number=} params.phi2 =0.] rotation of the 2nd point
* @param {number=} params.wMid =0.] local y deflection from continuous load in the middle of the beam
* @param {number=} params.realLen =1.] real length (in units of u and w) of beam
* @param {number=} params.scale =1.] scale parameter for the deflection line
* @param {boolean=} params.closedBegin =true] the line conecting 1st point with 1st nodal value
* @param {boolean=} params.closedEnd =true] the line conecting 2nd point with 2nd nodal value
* @param {boolean=} params.closedBase =true] the line conecting 1st and 2nd point
* @param {number=} params.nSeg =5] number of interpolation points
* @borrows Raphael.fn.CurveObj#point1 as this.point1
* @borrows Raphael.fn.CurveObj#point2 as this.point2
* @borrows Raphael.fn.CurveObj#scale as this.scale
* @borrows Raphael.fn.CurveObj#closedBegin as this.closedBegin
* @borrows Raphael.fn.CurveObj#closedEnd as this.closedEnd
* @borrows Raphael.fn.CurveObj#closedBase as this.closedBase
* @borrows Raphael.fn.CurveObj.setPathList as this.setPathList
* @borrows Raphael.fn.CurveObj.consts as this.consts
* @borrows Raphael.fn.CurveObj.update as this.update
* @property {number} w1 local y displacement of 1st point
* @property {number} phi1 rotation of 1st point
* @property {number} w2 local y displacement of 2nd point
* @property {number} phi2 rotation of 2nd point
* @property {number} realLen real length (in units of u and w) of beam
* @property {number} wMid local y deflection from continuous load in the middle of the beam
* @property {number} nSeg number of interpolation points
* @this Raphael.fn.Defl4thOrderNoU
* @constructor
*/
Raphael.fn.Defl4thOrderNoU = function(point1,point2,params) {
	params = params==undefined? {} : params;
	params.u1 = 0.;
	params.u2 = 0.;
	/** @alias Raphael.fn.Defl4thOrderNoU.prototype */
	var retDefl4thOrderNoU = this.DeflCurve(point1,point2,params);

	/** Sets new values of w and phi in both points + wMid
	* @param {number} w1 local y deflection in the 1st point
	* @param {number} phi1 rotation of the 1st point
	* @param {number} w2 local y deflection in the 2nd point
	* @param {number} phi2 rotation of the 2nd point
	* @param {number} wMid local y deflection from continuous load in the middle of the beam
	*/
	retDefl4thOrderNoU.setDspl = function(w1,phi1,w2,phi2,wMid) {
		this.setDsplFull(0,w1,phi1,0,w2,phi2,wMid);
	}

	retDefl4thOrderNoU.update();
	return retDefl4thOrderNoU;
}






/** Dimension
* @class represents dimension
* @param {Raphael.Paper} paper raphael instance
* @param {RaphaelTools.Point2d} point1 1st point
* @param {RaphaelTools.Point2d} point2 2nd point
* @param {{
				type:string,
				height:number,
				overhang:number,
				offset:number,
				textOffset:number,
				lower:boolean,
				val:(string|number),
				pathAttrs:Object,
				textAttrs:Object,
				arrowSize:number,
				threshold:number}=} params
* @param {string=} params.type ="c"] type fo arrow ("c" for civil engineering, "m" for mechanical engineering)
* @param {number=} params.height =30.] height
* @param {number=} params.overhang =6.] size of overhang
* @param {number=} params.offset =10.] offset
* @param {number=} params.textOffset =12.] text offset
* @param {boolean=} prams.lower =true] if the dimension is lower or upper from line point1-point2
* @param {string|number=} params.val =""] value to be displayed
* @param {Object=} prams.pathAttrs ={}] attrs passed to path
* @param {Object=} prams.textAttrs ={}] attrs passed to text
* @param {number=} prams.arrowSize =overhang] arrow size (only for "c" type)
* @param {number=} prams.threshold =1.] length threshold, if length is lower, the dimension is hidden
* @property {Point2d} point1 1st point
* @property {Point2d} point2 2nd point
* @property {string} type type fo arrow ("c" for civil engineering, "m" for mechanical engineering)
* @property {number} height height
* @property {number} overhang size of overhang
* @property {number} offset offset
* @property {number} textOffset text offset
* @property {string|number} val value to be displayed
* @property {Raphael.el} path path
* @property {Raphael.el} text text
* @property {Array} pathList path list
* @property {number} arrowSize arrow size
* @property {number} threshold length threshold
* @property {boolean} lower if the dimension is lower or upper
* @this RaphaelSM.Dimension
* @constructor
*/
RaphaelSM.Dimension = function(paper,point1,point2,params) {
	this.point1 = point1;
	this.point2 = point2;
	params = params==undefined? {} : params;
	this.type = params.type==undefined? "c" : params.type=="e"? "e" : "c";
	this.height = params.height==undefined? 30. : params.height;
	this.overhang = params.overhang==undefined? 6. : params.overhang;
	this.offset = params.offset==undefined? 10. : params.offset;
	this.arrowSize = params.arrowSize==undefined? this.overhang : params.arrowSize;
	this.textOffset = params.textOffset==undefined? 12. : params.textOffset;
	this.val = params.val==undefined? "" : params.val;
	this.threshold = params.threshold==undefined? 1. : params.threshold;
	this.lower = params.lower==undefined? true : params.lower;
	var pathAttrs = params.pathAttrs==undefined? {} : params.pathAttrs;
	this.path = paper.path().attr(pathAttrs);
	var textAttrs = params.textAttrs==undefined? {} : params.textAttrs;
	if (this.type == "e") { textAttrs["arrow-end"] = "classic-midium-long"; }
	this.text = paper.text(0,0,this.val).attr({"font-size":20}).attr(textAttrs);
	this.pathList = [];
}

/** Updates receiver
*/
RaphaelSM.Dimension.prototype.update = function() {
	var dx = this.point2.x - this.point1.x;
	var dy = this.point2.y - this.point1.y;
	var l = Math.sqrt(dx*dx+dy*dy);
	this.show();
	if (l < this.threshold) {
		this.hide();
		return;
	}
	var sign = this.lower? 1 : -1
	var c = dx/l;
	var s = dy/l;
	var y1 = sign*this.offset;
	var y2 = y1 + sign*this.height;
	var y3 = y2 + sign*this.overhang;
	var x = this.overhang;
	this.pathList = [
		["M",this.point1.x-s*y1,this.point1.y+c*y1],
		["L",this.point1.x-s*y3,this.point1.y+c*y3],
		["M",this.point2.x-s*y1,this.point2.y+c*y1],
		["L",this.point2.x-s*y3,this.point2.y+c*y3],
		["M",this.point1.x-s*y2,this.point1.y+c*y2],
		["L",this.point2.x-s*y2,this.point2.y+c*y2]
	];
	if (this.type == "c") {
		y1 = y2 + this.arrowSize;
		y3 = y2 - this.arrowSize;
		this.pathList.push(["M",this.point1.x-s*y1-c*x,this.point1.y+c*y1-s*x]);
		this.pathList.push(["L",this.point1.x-s*y3+c*x,this.point1.y+c*y3+s*x]);
		this.pathList.push(["M",this.point2.x-s*y1-c*x,this.point2.y+c*y1-s*x]);
		this.pathList.push(["L",this.point2.x-s*y3+c*x,this.point2.y+c*y3+s*x]);
	}
	this.path.attr({"path":this.pathList})
	//
	y3 = this.offset + this.height - sign*this.textOffset;
	x  = .5*(this.point1.x+this.point2.x) - sign*s*y3;
	y1 = .5*(this.point1.y+this.point2.y) + sign*c*y3;
	var a = Math.acos(c)*180./Math.PI;
	if (s<0) { a = 360. - a; }
	this.text.attr({"transform":"t"+x+","+y1+"r"+a,"text":this.val});
	this.text.attr({"x":+x,"y":y1,"transform":"r"+a,"text":this.val});
}

/** Sets new (text) value of receiver
* @param {string|number} val new text value
*/
RaphaelSM.Dimension.prototype.setVal = function(val) {
	this.val = val;
	this.update();
}

/** Hides receiver
*/
RaphaelSM.Dimension.prototype.hide = function() {
	this.path.hide();
	this.text.hide();
}

/** Shows receiver
*/
RaphaelSM.Dimension.prototype.show = function() {
	this.path.show();
	this.text.show();
}

/** Creates new Dimension object, for parameters meaning see {@link RaphaelSM.Dimension}
* @returns {RaphaelSM.Dimension} new Dimension object
*/
RaphaelSM.Dimension.create = function(paper,point1,point2,params) {
	var ret = new RaphaelSM.Dimension(paper,point1,point2,params);
	ret.update();
	return ret;
}






/** Spring [GeomObj]
* @class represents elastic spring
* @extends Raphael.fn.GeomObj
* @borrows Raphael.fn.GeomObj.copy as this.copy
* @borrows Raphael.fn.GeomObj.cos as this.cos
* @borrows Raphael.fn.GeomObj.sin as this.sin
* @borrows Raphael.fn.GeomObj.setX as this.setX
* @borrows Raphael.fn.GeomObj.setY as this.setY
* @borrows Raphael.fn.GeomObj.setXY as this.setXY
* @borrows Raphael.fn.GeomObj.translateX as this.translateX
* @borrows Raphael.fn.GeomObj.translateY as this.translateY
* @borrows Raphael.fn.GeomObj.translateXY as this.translateXY
* @borrows Raphael.fn.GeomObj.rotate as this.rotate
* @borrows Raphael.fn.GeomObj.setAngle this.setAngle
* @borrows Raphael.fn.GeomObj.update as this.update
* @borrows Raphael.fn.GeomObj#x as this.x
* @borrows Raphael.fn.GeomObj#y as this.y
* @borrows Raphael.fn.GeomObj#size as this.size
* @borrows Raphael.fn.GeomObj#angle as this.angle
* @borrows Raphael.fn.GeomObj#pathList as this.pathList
* @param {Object} params parameters, see below
* @param {number=} params.x =0.] x coordinate of spring start [pixel]
* @param {number=} params.y =0.] y coordinate of spring start [pixel]
* @param {number=} params.size =200.] length of the spring [pixel]
* @param {number=} params.width =40.] width of the spring [pixel]
* @param {number=} params.angle =0.] the spring will be rotated by angle [deg]
* @param {number=} params.offset =20.] gap between spring start and actual spring [pixel]
* @param {number=} params.num =14] number of peaks
* @property {number} width width of the spring [pixel]
* @property {number} offset gap between spring start and actual spring [pixel]
* @property {number} num number of peaks
* @this Raphael.fn.Spring
* @constructor
*/
Raphael.fn.Spring = function(params) {
	params = params==undefined? {} : params;
	/** @alias Raphael.fn.Spring.prototype */
	var retSpring = this.GeomObj(params).attr({"stroke-linejoin":"miter","stroke-miterlimit":1000});

	if (params.size==undefined) { retSpring.size = 100.; }
	retSpring.size = params.size==undefined? 200 : params.size;
	retSpring.width = params.width==undefined? 40 : params.width;
	retSpring.offset = params.offset==undefined? 20 : params.offset;
	retSpring.num = params.num==undefined? 14 : params.num;

	/** Sets svg path of receiver
	*/
	retSpring.setPathList = function() {
		var sin = this.sin();
		var cos = this.cos();
		var p = new Array(2*this.num+3);
		p[0] = ["M",this.x,this.y];
		p[1] = ["l",this.offset*cos,this.offset*sin];
		var dx=(this.size-2*this.offset) / this.num * .5;
		var w;
		for (var i=0; i<this.num; i++) {
			w = .5*this.width*Math.pow(-1,i);
			p[2+2*i] = ["l",dx*cos-w*sin,dx*sin+w*cos];
			p[2+2*i+1] = ["l",dx*cos+w*sin,dx*sin-w*cos];
		}
		p[2*this.num+2] = ["L",this.x+cos*this.size,this.y+sin*this.size];
		this.pathList = p;
	}

	/** Returns copy of receiver
	* @returns {Raphael.fn.Spring} copy of receiver
	*/
	retSpring.makeCopy = function() {
		var ret2 = this.paper.Spring({x:this.x,y:this.y,size:this.size,angle:this.angle,width:this.width,offset:this.offset,num:this.num});
		ret2.attr(this.attr());
		return ret2;
	}

	retSpring.resize = function(s) {
		this.size = s;
		this.update();
	}

	retSpring.update();
	return retSpring;
}





/** Dashpot [GeomObj]
* @class represents viscous dashpot
* @extends Raphael.fn.GeomObj
* @borrows Raphael.fn.GeomObj.copy as this.copy
* @borrows Raphael.fn.GeomObj.cos as this.cos
* @borrows Raphael.fn.GeomObj.sin as this.sin
* @borrows Raphael.fn.GeomObj.setX as this.setX
* @borrows Raphael.fn.GeomObj.setY as this.setY
* @borrows Raphael.fn.GeomObj.setXY as this.setXY
* @borrows Raphael.fn.GeomObj.translateX as this.translateX
* @borrows Raphael.fn.GeomObj.translateY as this.translateY
* @borrows Raphael.fn.GeomObj.translateXY as this.translateXY
* @borrows Raphael.fn.GeomObj.rotate as this.rotate
* @borrows Raphael.fn.GeomObj.setAngle this.setAngle
* @borrows Raphael.fn.GeomObj.update as this.update
* @borrows Raphael.fn.GeomObj#x as this.x
* @borrows Raphael.fn.GeomObj#y as this.y
* @borrows Raphael.fn.GeomObj#size as this.size
* @borrows Raphael.fn.GeomObj#angle as this.angle
* @borrows Raphael.fn.GeomObj#pathList as this.pathList
* @param {Object} params parameters, see below
* @param {number=} params.x =0.] x coordinate of dashpot start [pixel]
* @param {number=} params.y =0.] y coordinate of dashpot start [pixel]
* @param {number=} params.size =200.] length of the dashpot [pixel]
* @param {number=} params.width =40.] width of the dashpot [pixel]
* @param {number=} params.angle =0.] the dashpot will be rotated by angle [deg]
* @param {number=} params.offset =20.] gap between dashpot start and actual dashpot [pixel]
* @param {number=} params.dspl =0] displacement from central position
* @param {number=} params.dw =5] transverse gap
* @property {number} width width of the dashpot [pixel]
* @property {number} offset gap between dashpot start and actual dashpot [pixel]
* @property {number} dspl displacement
* @property {number} dw transverse gap
* @this Raphael.fn.Dashpot
* @constructor
*/
Raphael.fn.Dashpot = function(params) {
	params = params==undefined? {} : params;
	/** @alias Raphael.fn.Dashpot.prototype */
	var retDashpot = this.GeomObj(params);

	if (params.size==undefined) { retDashpot.size = 100.; }
	retDashpot.size = params.size==undefined? 200 : params.size;
	retDashpot.width = params.width==undefined? 40 : params.width;
	retDashpot.offset = params.offset==undefined? 20 : params.offset;
	retDashpot.dspl = params.dspl==undefined? 0. : params.dspl;
	retDashpot.dw = params.dw==undefined? 5 : params.dw;

	/** Sets svg path of receiver
	*/
	retDashpot.setPathList = function() {
		var sin = this.sin();
		var cos = this.cos();
		var ds =  .66667*(this.size - this.offset);
		var w2 = .5*this.width;
		this.pathList = [
			["M",this.x,this.y],
			["l",this.offset*cos,this.offset*sin],
			["l",-w2*sin,w2*cos],
			["l",ds*cos,ds*sin],
			["M",this.x+this.offset*cos,this.y+this.offset*sin],
			["l",+w2*sin,-w2*cos],
			["l",ds*cos,ds*sin],
			["M",this.x + cos*(this.offset+.5*ds+this.dspl) - sin*(w2-this.dw), this.y + sin*(this.offset+.5*ds+this.dspl) + cos*(w2-this.dw)],
			["l",2*(w2-this.dw)*sin,-2*(w2-this.dw)*cos],
			["M",this.x + cos*(this.offset+.5*ds+this.dspl), this.y + sin*(this.offset+.5*ds+this.dspl)],
			["l",ds*cos,ds*sin]
		];
	}

	/** Returns copy of receiver
	* @returns {Raphael.fn.Dashpot} copy of receiver
	*/
	retDashpot.makeCopy = function() {
		var ret2 = this.paper.Dashpot({x:this.x,y:this.y,size:this.size,angle:this.angle,width:this.width,offset:this.offset,dspl:this.dspl,dw:this.dw});
		ret2.attr(this.attr());
		return ret2;
	}

	retDashpot.resize = function(s) {
		this.size = s;
		this.update();
	}

	retDashpot.update();
	return retDashpot;
}





/** PlasticElement [GeomObj]
* @class represents plastic element
* @extends Raphael.fn.GeomObj
* @borrows Raphael.fn.GeomObj.copy as this.copy
* @borrows Raphael.fn.GeomObj.cos as this.cos
* @borrows Raphael.fn.GeomObj.sin as this.sin
* @borrows Raphael.fn.GeomObj.setX as this.setX
* @borrows Raphael.fn.GeomObj.setY as this.setY
* @borrows Raphael.fn.GeomObj.setXY as this.setXY
* @borrows Raphael.fn.GeomObj.translateX as this.translateX
* @borrows Raphael.fn.GeomObj.translateY as this.translateY
* @borrows Raphael.fn.GeomObj.translateXY as this.translateXY
* @borrows Raphael.fn.GeomObj.rotate as this.rotate
* @borrows Raphael.fn.GeomObj.setAngle this.setAngle
* @borrows Raphael.fn.GeomObj.update as this.update
* @borrows Raphael.fn.GeomObj#x as this.x
* @borrows Raphael.fn.GeomObj#y as this.y
* @borrows Raphael.fn.GeomObj#size as this.size
* @borrows Raphael.fn.GeomObj#angle as this.angle
* @borrows Raphael.fn.GeomObj#pathList as this.pathList
* @param {Object} params parameters, see below
* @param {number=} params.x =0.] x coordinate of plastic element start [pixel]
* @param {number=} params.y =0.] y coordinate of plastic element start [pixel]
* @param {number=} params.size =200.] length of the plastic element [pixel]
* @param {number=} params.width =40.] width of the plastic element [pixel]
* @param {number=} params.angle =0.] the plastic element will be rotated by angle [deg]
* @param {number=} params.offset =20.] gap between plastic element start and actual plastic element [pixel]
* @param {number=} params.dspl =0] displacement from central position
* @property {number} width width of the plastic element [pixel]
* @property {number} offset gap between plastic element start and actual plastic element [pixel]
* @property {number} dspl displacement
* @this Raphael.fn.PlasticElement
* @constructor
*/
Raphael.fn.PlasticElement = function(params) {
	params = params==undefined? {} : params;
	/** @alias Raphael.fn.PlasticElement.prototype */
	var retPlasticElement = this.GeomObj(params);

	if (params.size==undefined) { retPlasticElement.size = 100.; }
	retPlasticElement.size = params.size==undefined? 200 : params.size;
	retPlasticElement.width = params.width==undefined? 10 : params.width;
	retPlasticElement.offset = params.offset==undefined? 10 : params.offset;
	retPlasticElement.dspl = params.dspl==undefined? 0. : params.dspl;

	/** Sets svg path of receiver
	*/
	retPlasticElement.setPathList = function() {
		var sin = this.sin();
		var cos = this.cos();
		var s = this.size
		var ds =  .66667*(s - this.offset);
		var w = this.width;
		var cc = .7;
		this.pathList = [
			["M",this.x,this.y],
			["l",ds*cos,ds*sin],
			["l",cos*cc*(-w) - sin*w, sin*cc*(-w) + cos*w],
			["l",cos*cc*(-w) - sin*(-w), sin*cc*(-w) + cos*(-w)],
			["M",this.x+(s+this.dspl)*cos,this.y+(s+this.dspl)*sin],
			["l",cos*(-this.offset),sin*(-this.offset)],
			["m",0,0],
			["l",cos*cc*(-w) - sin*w, sin*cc*(-w) + cos*w],
			["m",0,0],
			["l",cos*(-ds), sin*(-ds)]
		];
	}

	/** Returns copy of receiver
	* @returns {Raphael.fn.PlasticElement} copy of receiver
	*/
	retPlasticElement.makeCopy = function() {
		var ret2 = this.paper.PlasticElement({x:this.x,y:this.y,size:this.size,angle:this.angle,width:this.width,offset:this.offset,dspl:this.dspl,dw:this.dw});
		ret2.attr(this.attr());
		return ret2;
	}

	retPlasticElement.resize = function(s) {
		this.size = s;
		this.update();
	}

	retPlasticElement.update();
	return retPlasticElement;
}
